""" Python Application Launcher
Copyright (C) 2002 James ''Wez''Weatherall

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""

import sys
def _stderr(text):
    sys.stdout.write("pylau:%s\n" % text)
if "--no-pylau" in sys.argv:
    _stderr("pylau disabled")
    sys.argv.remove("--no-pylau")
    raise ImportError

# - We will only be imported once per interpreter
#   This means we ALWAYS check for an existing instance
#   of pylau and run one if none is found

import socket
import os

# - Create the pylau socket dir if required
sockdir = os.path.join("/tmp",".pylau")
if not os.access(sockdir, 0):
    os.mkdir(sockdir)
    os.chmod(sockdir, 0777)

# - Attempt to connect to an existing launcher
conn = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
try:
    pipename = os.path.join(sockdir, os.environ["DISPLAY"])
except:
    _stderr("no X11 display")
    raise ImportError

try:
    # - Connect to pylau to launch the host script
    _stderr("locating pylau")
    conn.connect(pipename)
    _stderr("connected to server at %s" % pipename)

    try:
        sys.argv[0] = os.path.abspath(sys.argv[0])
        conn.send(repr(sys.argv)+"\n")
        conn.send(repr(os.environ)+"\n")
        f=conn.makefile()
        if eval(f.readline()):
            _stderr("success")
            sys.exit()
    except socket.error:
        _stderr("failure")
        raise ImportError

except socket.error:
    # - There is no existing pylau instance - start one
    _stderr("connect error: %s" % sys.exc_info()[1])

    # - Kill an existing, duff, socket
    try: os.unlink(pipename)
    except: pass

    # - Open the socket to listen for requests on
    try:
        conn = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        conn.bind(pipename)
        conn.listen(5)
    except:
        _stderr("unable to bind socket")
        raise ImportError

    # - Import Gtk
    import gtk

    _test_env = ["USERNAME", "ENV", "PYTHONPATH", "HOSTTYPE",
                 "LOGNAME", "USER", "INPUTRC", "HOME", "PATH",
                 "LD_LIBRARY_PATH", "DISPLAY", "HOSTNAME"]

    # - Internal functions to launch apps correctly(!)
    def _launch(conn, info):
        try:
            sock = conn.accept()[0]
            file = sock.makefile()
            argv = eval(file.readline())
            environ = eval(file.readline())
            for key in _test_env:
                if ((environ.has_key(key) and os.environ.has_key(key)) and
                    (environ[key] != os.environ[key])):
                    sock.send(repr(0)+"\n")
                    return
            _stderr("launch %s" % str(argv))
        except:
            _stderr("error reading request")
            sock.send(repr(0)+"\n")
            return

        try:
            import imp
            path, name = os.path.split(argv[0])
            name, ext = os.path.splitext(name)
            if path: paths = [path]
            else: paths = string.split(os.environ["PATH"],":")
            fd, pathname, desc = imp.find_module(name, paths)
            app = imp.load_module(name, fd, pathname, desc)
            try:
                app.Main(argv)
            except:
                _stderr("script %s has no Main(argv)" % name)
                sock.send(repr(0)+"\n")
                return
        except:
            _stderr("unable to launch:%s:%s" % (sys.exc_info()[0], sys.exc_info()[1]))
            sock.send(repr(0)+"\n")
            return

        try: sock.send(repr(1)+"\n")
        except: pass

    # *** Dodgy GTK hacking
    def _events_pending():
        if _gtk_mainloop_count: return 0
        return _gtk_events_pending()
    def _mainiteration():
        if _gtk_mainloop_count: return 0
        return _gtk_mainiteration()

    # *** Safer GTK hacking
    def _mainloop():
        global _gtk_mainloop_count
        old = _gtk_mainloop_count
        _gtk_mainloop_count = _gtk_mainloop_count + 1
        if not old: _gtk_mainloop()

    def _mainquit(*args):
        global _gtk_mainloop_count
        _gtk_mainloop_count = _gtk_mainloop_count - 1
        if not _gtk_mainloop_count: _gtk_mainquit()

    # - Rewire gtk... !!!
    _gtk_mainloop_count = 0
    _gtk_mainloop = gtk.mainloop
    _gtk_mainquit = gtk.mainquit

    _gtk_mainiteration=gtk.mainiteration
    _gtk_events_pending=gtk.events_pending
    gtk.mainiteration=_mainiteration
    gtk.events_pending=_events_pending

    gtk.mainloop = _mainloop
    gtk.mainquit = _mainquit

    # - Hook us in to Gtk's select() calls
    gtk.input_add(conn, gtk.GDK.INPUT_READ, _launch)
